"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferSols = exports.transferSpl = exports.SolanaSwapClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const RaydiumSwap_1 = __importDefault(require("../server/RaydiumSwap"));
const utils_1 = require("../utils");
const helpers_1 = require("@solana-developers/helpers");
const anchor_1 = require("@coral-xyz/anchor");
const MAX_RETRIES = 20;
class SolanaSwapClient {
    constructor(poolKeys, rpc = 'https://api.mainnet-beta.solana.com') {
        this.rpc = 'https://api.mainnet-beta.solana.com';
        this.rpc = rpc;
        this.connection = new web3_js_1.Connection(this.rpc);
        if (!poolKeys || poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
        this.poolKeys = poolKeys;
        if (!this.poolKeys || this.poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
    }
    swap(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null, fee = null, feePercent, feeRecipient) {
            var _a, _b, _c, _d, _e, _f, _g;
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const transactionInfo = yield raydiumSwap.getSwapTransaction(tokenOut, amountInSols, poolInfo, slippage, priceImpact, fee, feePercent, feeRecipient);
            const txid = yield raydiumSwap.sendLegacyTransaction(transactionInfo.transaction, MAX_RETRIES);
            const latestBlockhash = yield this.connection.getLatestBlockhash();
            yield this.connection.confirmTransaction({
                signature: txid,
                blockhash: latestBlockhash.blockhash,
                lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
            }, "confirmed");
            const tx = yield this.connection.getTransaction(txid, { "commitment": "confirmed", "maxSupportedTransactionVersion": 1 });
            let amountOut = 0;
            if (tokenOut === "So11111111111111111111111111111111111111112") {
                amountOut = transactionInfo.amountOut;
            }
            else {
                const preTokenOutBalance = (_b = (_a = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _a === void 0 ? void 0 : _a.preTokenBalances) === null || _b === void 0 ? void 0 : _b.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
                const postTokenOutBalance = (_d = (_c = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _c === void 0 ? void 0 : _c.postTokenBalances) === null || _d === void 0 ? void 0 : _d.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
                const postBalance = new anchor_1.BN((_e = postTokenOutBalance === null || postTokenOutBalance === void 0 ? void 0 : postTokenOutBalance.uiTokenAmount.amount) !== null && _e !== void 0 ? _e : "0");
                const preBalance = new anchor_1.BN((_f = preTokenOutBalance === null || preTokenOutBalance === void 0 ? void 0 : preTokenOutBalance.uiTokenAmount.amount) !== null && _f !== void 0 ? _f : "0");
                const decimals = (_g = postTokenOutBalance === null || postTokenOutBalance === void 0 ? void 0 : postTokenOutBalance.uiTokenAmount.decimals) !== null && _g !== void 0 ? _g : 0;
                amountOut = postBalance.sub(preBalance).toNumber() / Math.pow(10, decimals);
            }
            return {
                txid,
                amountOut,
                fee: transactionInfo.fee
            };
        });
    }
    calcAmountOut(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null) {
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const directionIn = poolInfo.quoteMint.toString() == tokenOut;
            const amountOut = yield raydiumSwap.calcAmountOut(poolInfo, amountInSols, directionIn, slippage, priceImpact);
            const amountOutParsed = amountOut.amountOut.raw.toNumber() /
                Math.pow(10, amountOut.amountOut.currency.decimals);
            return {
                amountIn: amountInSols,
                amountOut: amountOutParsed,
            };
        });
    }
}
exports.SolanaSwapClient = SolanaSwapClient;
function transferSpl(connection_1, fromPrivKeyBs58_1, token_1, amount_1, receiver_1) {
    return __awaiter(this, arguments, void 0, function* (connection, fromPrivKeyBs58, token, amount, receiver, fee = null) {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        let amountUi = (0, utils_1.convertNumber)(amount);
        const fromAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), from.publicKey);
        const toAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), to);
        const tx = new web3_js_1.Transaction();
        try {
            const toAccountInfo = yield connection.getAccountInfo(toAssociatedTokenAccount);
            if (!toAccountInfo) {
                tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
            }
        }
        catch (e) {
            tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
        }
        // Fake instruction to calculate compute unit
        tx.add((0, spl_token_1.createTransferInstruction)(fromAssociatedTokenAccount, toAssociatedTokenAccount, from.publicKey, 1));
        // Calculate the transaction fee
        let transferFee = utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL;
        // Get temporary units
        tx.instructions.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
            microLamports: utils_1.MICRO_LAMPORTS_PER_LAMPORTS
        }));
        const units = yield (0, helpers_1.getSimulationComputeUnits)(connection, tx.instructions, from.publicKey, []);
        tx.instructions.shift(); // remove fake setComputeUnitPrice
        tx.instructions.pop(); // remove fake transfer instruction
        let computeUnitPrice = 0;
        if (fee && units) {
            transferFee = (0, utils_1.convertNumber)(fee);
            if (transferFee >= utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL) {
                const diffFee = transferFee - utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL;
                computeUnitPrice = Math.floor((diffFee * web3_js_1.LAMPORTS_PER_SOL / units) * utils_1.MICRO_LAMPORTS_PER_LAMPORTS);
            }
            else {
                throw Error("Fee must be greater than 5000 lamports (0.000005 SOL)");
            }
        }
        const balanceInSol = (0, utils_1.convertNumber)(((yield connection.getBalance(from.publicKey)) / web3_js_1.LAMPORTS_PER_SOL).toString());
        if (transferFee > balanceInSol) {
            throw Error("Fee is greater than balance");
        }
        const fromTokenBalance = (yield (0, spl_token_1.getAccount)(connection, fromAssociatedTokenAccount)).amount.toString();
        const decimals = Math.pow(10, (yield (0, spl_token_1.getMint)(connection, (0, utils_1.convertPubkey)(token))).decimals);
        const balanceUi = (0, utils_1.convertNumber)(fromTokenBalance) / decimals;
        if (amountUi === 0) {
            amountUi = balanceUi;
            if (!units) {
                throw Error("Cannot send max balance");
            }
        }
        tx.add((0, spl_token_1.createTransferInstruction)(fromAssociatedTokenAccount, toAssociatedTokenAccount, from.publicKey, amountUi * decimals));
        if (computeUnitPrice > 0 && units) {
            tx.instructions.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: computeUnitPrice
            }));
            tx.instructions.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                units
            }));
        }
        tx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
        tx.feePayer = from.publicKey;
        const recoverTx = web3_js_1.Transaction.from(tx.serialize({ requireAllSignatures: false }));
        // transaction sign
        recoverTx.sign(from);
        const signature = yield connection.sendRawTransaction(recoverTx.serialize());
        return signature;
    });
}
exports.transferSpl = transferSpl;
function transferSols(connection_1, fromPrivKeyBs58_1, amount_1, receiver_1) {
    return __awaiter(this, arguments, void 0, function* (connection, fromPrivKeyBs58, amount, receiver, fee = null) {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        let amountInSols = (0, utils_1.convertNumber)(amount);
        // Calculate the transaction fee
        let transferFee = utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL;
        const units = 500; // compute unit limit to send SOL
        let computeUnitPrice = 0;
        if (fee) {
            transferFee = (0, utils_1.convertNumber)(fee);
            if (transferFee >= utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL) {
                const diffFee = transferFee - utils_1.MINIMUM_TRANSACTION_FEE_IN_SOL;
                computeUnitPrice = Math.floor((diffFee * web3_js_1.LAMPORTS_PER_SOL / units) * utils_1.MICRO_LAMPORTS_PER_LAMPORTS);
            }
            else {
                throw Error("Fee must be greater than 5000 lamports (0.000005 SOL)");
            }
        }
        const balanceInSol = (0, utils_1.convertNumber)(((yield connection.getBalance(from.publicKey)) / web3_js_1.LAMPORTS_PER_SOL).toString());
        // if amount = 0, set the `amountInSols` as all SOL balance of `from` subtract transfer fee
        if (amountInSols === 0) {
            if (transferFee > balanceInSol) {
                throw Error("Fee is greater than balance");
            }
            amountInSols = balanceInSol - transferFee;
        }
        else if (amountInSols > balanceInSol) {
            throw Error("Insufficient balance");
        }
        const transferInstruction = [
            web3_js_1.SystemProgram.transfer({
                fromPubkey: from.publicKey,
                toPubkey: to,
                lamports: amountInSols * web3_js_1.LAMPORTS_PER_SOL
            }),
        ];
        if (computeUnitPrice > 0) {
            transferInstruction.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: computeUnitPrice
            }));
            transferInstruction.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                units
            }));
        }
        let { blockhash } = yield connection.getLatestBlockhash();
        const message = new web3_js_1.TransactionMessage({
            payerKey: from.publicKey,
            recentBlockhash: blockhash,
            instructions: transferInstruction,
        }).compileToV0Message();
        const transaction = new web3_js_1.VersionedTransaction(message);
        transaction.sign([from]);
        const signature = yield connection.sendRawTransaction(transaction.serialize());
        return signature;
    });
}
exports.transferSols = transferSols;
