"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const bodyParser = require("body-parser");
const dotenv = require("dotenv");
dotenv.config();
const web3_js_1 = require("@solana/web3.js");
const RaydiumSwap_1 = __importStar(require("./RaydiumSwap"));
const transfer_1 = require("./transfer");
const validate_1 = require("./validate");
const swap_1 = require("./swap");
const liquidityFile = process.env.LIQUIDITY_FILE ||
    "https://api.raydium.io/v2/sdk/liquidity/mainnet.json";
const rpc = process.env.RPC_URL || "https://api.mainnet-beta.solana.com";
const maxRetries = 20;
console.log(`RPC_URL: ${rpc}`);
const connection = new web3_js_1.Connection(rpc);
let poolKeys = undefined;
const loadPoolKeysAsync = () => __awaiter(void 0, void 0, void 0, function* () {
    if (!poolKeys || poolKeys.length === 0) {
        poolKeys = yield (0, RaydiumSwap_1.loadPoolKeys)(liquidityFile);
    }
    return poolKeys;
});
loadPoolKeysAsync();
const app = express();
app.use(bodyParser.json());
const port = process.env.PORT || 8000;
app.post("/swap", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { tokenA, tokenB, amountIn, direction, ownerPrivKeyBs58, slippage, priceImpact, fee, feePercent, feeRecipient } = req.body;
        const amountInSols = (0, validate_1.convertNumber)(amountIn);
        const owner = (0, validate_1.convertKeypair)(ownerPrivKeyBs58);
        const _poolKeys = yield loadPoolKeysAsync();
        if (!_poolKeys || _poolKeys.length === 0) {
            throw new Error("Pool keys not loaded");
        }
        const raydiumSwap = new RaydiumSwap_1.default(rpc, owner, _poolKeys);
        const txid = yield (0, swap_1.swap)({
            raydiumSwap,
            tokenA,
            tokenB,
            amountIn: amountInSols,
            direction,
            maxRetries,
            slippage,
            priceImpact,
            fee,
            feePercent,
            feeRecipient,
        });
        res.send({
            status: "ok",
            data: {
                txid,
            },
        });
    }
    catch (error) {
        console.error(error);
        res.status(500).send({
            status: "error",
            error: JSON.stringify(error),
        });
    }
}));
app.post("/calcAmountOut", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { tokenA, tokenB, amountIn, direction } = req.body;
        const amountInSols = (0, validate_1.convertNumber)(amountIn);
        const tmpOwner = web3_js_1.Keypair.generate();
        const _poolKeys = yield loadPoolKeysAsync();
        if (!_poolKeys || _poolKeys.length === 0) {
            throw new Error("Pool keys not loaded");
        }
        const raydiumSwap = new RaydiumSwap_1.default(rpc, tmpOwner, _poolKeys);
        const result = yield (0, swap_1.calcAmountOut)({
            raydiumSwap,
            tokenA,
            tokenB,
            amountIn: amountInSols,
            direction,
        });
        res.send({
            status: "ok",
            data: result,
        });
    }
    catch (error) {
        console.error(error);
        res.status(500).send({
            status: "error",
            error: JSON.stringify(error),
        });
    }
}));
app.post("/transferSols", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { fromPrivKeyBs58, toPubkey, sols } = req.body;
        const to = (0, validate_1.convertPubkey)(toPubkey);
        const from = (0, validate_1.convertKeypair)(fromPrivKeyBs58);
        const amountSols = (0, validate_1.convertNumber)(sols);
        const params = {
            from,
            to,
            sols: amountSols,
        };
        const txId = yield (0, transfer_1.transferSols)({
            connection,
            params,
        });
        res.send({
            status: "ok",
            data: {
                txId,
            },
        });
    }
    catch (error) {
        console.error(error);
        res.status(500).send({
            status: "error",
            error: JSON.stringify(error),
        });
    }
}));
app.get("/balance", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { pubkey } = req.query;
        const publicKey = (0, validate_1.convertPubkey)(pubkey);
        const balance = yield connection.getBalance(publicKey);
        res.send({
            status: "ok",
            data: {
                balance: (0, transfer_1.lamportsToSol)(balance),
            },
        });
    }
    catch (error) {
        console.error(error);
        res.status(500).send({
            status: "error",
            error: JSON.stringify(error),
        });
    }
}));
app.listen(port, () => {
    console.log(`[server]: Server is running at http://localhost:${port}`);
});
