"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferSols = exports.withDecimalPoint = exports.tokenBalanceToNumber = exports.lamportsToSol = exports.solToLamports = void 0;
const web3_js_1 = require("@solana/web3.js");
const anchor_1 = require("@coral-xyz/anchor");
const SOL_DECIMALS = 9;
const solToLamports = (amountSol) => {
    return new anchor_1.BN(amountSol.toFixed(SOL_DECIMALS).replace(".", ""));
};
exports.solToLamports = solToLamports;
const lamportsToSol = (lamports) => {
    return tokenBalanceToNumber(lamports, SOL_DECIMALS);
};
exports.lamportsToSol = lamportsToSol;
function tokenBalanceToNumber(bn, decimals) {
    return Number(withDecimalPoint(bn, decimals));
}
exports.tokenBalanceToNumber = tokenBalanceToNumber;
function withDecimalPoint(bn, decimals) {
    const s = bn.toString().padStart(decimals + 1, "0");
    const l = s.length;
    return s.slice(0, l - decimals) + "." + s.slice(-decimals);
}
exports.withDecimalPoint = withDecimalPoint;
const transferSols = (_a) => __awaiter(void 0, [_a], void 0, function* ({ connection, params, }) {
    // convert SOls to lamports
    const { from, to, sols } = params;
    const lamports = (0, exports.solToLamports)(sols);
    const transaction = new web3_js_1.Transaction().add(web3_js_1.SystemProgram.transfer({
        fromPubkey: from.publicKey,
        toPubkey: to,
        lamports,
    }));
    const txId = yield connection.sendTransaction(transaction, [from]);
    return txId;
});
exports.transferSols = transferSols;
